function [M, DL, DNL_diag, DNL_cross] = model_params3DOF_NLdamp(x)
%% Model parameters
% Model: M*nu_dot + DL*nu + DNL(nu)*nu = tau

m = 8.6; %weight in air
Iz = 92502.96*10^(-6);  % [kgm^2]

MRB_sim = [m     0    0;  
           0     m    0;
           0     0    Iz];

% Added mass
MA_sim = diag([5.0899 19.4 0.16]);

% Linear damping
Xu = -x(1);
Yv = -x(2);
Yr = -x(3);
Nv = -x(3);
Nr = -x(4);

DL = [ -Xu    0.0  0.0; 
        0.0  -Yv  -Yr; 
        0.0  -Nv  -Nr]; 

Xuu = -x(5);
Yvv = -x(6);
Yrv = -x(7);
Yvr = -x(8);
Nrv = -x(9);
Nvr = -x(10);
Nrr = -x(11);

% Quadric damping
DNL_diag = diag([-Xuu -Yvv -Nrr]);
% Cross-terms
DNL_cross = [0.0  0.0  0.0; 
             0.0 -Yrv -Yvr;
             0.0 -Nrv -Nvr];

% Total mass matrix
M = MA_sim + MRB_sim;


end

