function [eta_est, nu_est, etadot_est, time_vec, tau_data] = sim3DOFdiscrete(x, eta_data, nu_data, drone_table, samp_time)
%% Thrust input
len      = length(nu_data);
tau_data = [drone_table.Controller_force_surge';...
            drone_table.Controller_force_sway'; ...
            drone_table.Controller_force_yaw'];
eta0     = eta_data(:,1);
eta0(3)  = rad2radpi(eta0(3));
nu0      = [0;0;0]; %endre?

%% Simulation parameters
time_vec = zeros(1,len+1);
eta_est  = zeros(3,len+1); eta_est(:,1) = eta0;
nu_est   = zeros(3,len+1); nu_est(:,1)  = nu0;
etadot_est = zeros(3,len+1); etadot_est(:,1) = rot_z(eta0(3))*nu0;

%% P2-drone ROV simulator parameters
nx = length(x);
switch nx
    case 4 
        [M, DL] = model_params3DOF_Ldamp(x);
    case 11
        [M, DL, DNL_diag, DNL_cross] = model_params3DOF_NLdamp(x);
    otherwise
        [M, DL] = model_params3DOF_Ldamp(x);
end
% M_inv = inv(M);

%%
eta_hat = eta0;
nu_hat  = nu0;

% Start discrete simulation
for ii = 1:len
    eta = eta_data(:,ii);
    nu  = nu_data(:,ii);
    tau = tau_data(:,ii);

    switch nx
        case 4 % Only linear damping
            nudot = M\(tau - DL*nu_hat);
        case 11 % Nonlinear damping and Coriolis
            DNL_cross_nu = [ 0.0; 
                 DNL_cross(2,2)*abs(nu_hat(3))*nu_hat(2) + DNL_cross(2,3)*abs(nu_hat(2))*nu_hat(3);
                 DNL_cross(3,2)*abs(nu_hat(3))*nu_hat(2) + DNL_cross(3,3)*abs(nu_hat(2))*nu_hat(3) ];
            Coriolis_nu = [-M(2,2)*nu_hat(2)*nu_hat(3); 
                  M(1,1)*nu_hat(1)*nu_hat(3);
                  M(2,2)*nu_hat(2)*nu_hat(1)- M(1,1)*nu_hat(1)*nu_hat(2)]; 
            nudot = M\(tau - DL*nu_hat - DNL_diag*abs(nu_hat).*nu_hat ...
                 -DNL_cross_nu - Coriolis_nu);
                 
        otherwise
            nudot = M\(tau - DL*nu_hat);
    end        
    
    psi = eta_hat(3);
    R = [cos(psi) -sin(psi) 0;
         sin(psi)  cos(psi) 0;
           0         0      1];

    eta_hat_next    = eta_hat + samp_time*(R*nu_hat); 
    eta_hat_next(3) = rad2radpi(eta_hat_next(3));
    nu_hat_next     = nu_hat  + samp_time*nudot;

    eta_est(:,ii+1) = eta_hat_next;
    nu_est(:,ii+1)  = nu_hat_next;
    
    psi = eta_hat_next(3);
    R = [cos(psi) -sin(psi) 0;
         sin(psi)  cos(psi) 0;
           0         0      1];

    etadot_est(:,ii+1) = R*nu_hat_next;
    time_vec(ii+1)  = samp_time*ii;

    eta_hat = eta_hat_next;
    nu_hat  = nu_hat_next;
end

