function [eta_est, nu_est, etadot_est, b_est, time_vec,tau_data] = sim3DOFdiscrete_obs(x, eta_data, nu_data, drone_table, samp_time)
%% Thrust input
%[len,~] = size(drone_table);
len      = length(nu_data);
tau_data = [drone_table.Controller_force_surge';...
            drone_table.Controller_force_sway'; ...
            drone_table.Controller_force_yaw'];
eta0     = eta_data(:,1);
eta0(3)  = rad2radpi(eta0(3));
nu0      = [0;0;0]; %endre?

%% Simulation parameters
time_vec = zeros(1,len+1);
b_est    = zeros(3,len+1); 
eta_est  = zeros(3,len+1); eta_est(:,1) = eta0;
nu_est   = zeros(3,len+1); nu_est(:,1)  = nu0;
etadot_est = zeros(3,len+1); etadot_est(:,1) = rot_z(eta0(3))*nu0;

%% P2-drone ROV simulator parameters
nx = length(x);
switch nx
    case 4 
        [M, DL] = model_params3DOF_Ldamp(x);
    case 11
        [M, DL, DNL_diag, DNL_cross] = model_params3DOF_NLdamp(x);
    otherwise
        [M, DL] = model_params3DOF_Ldamp(x);
end
% M_inv = inv(M);

%%
b_hat   = zeros(3,1);
eta_hat = eta0;
nu_hat  = nu0;

Tb  = 10*diag([1,1,180/pi]); %Tbinv = inv(Tb);
L1eta  = 0.1*diag([1,1,pi/180]);
L2eta  = 100*diag([1,1,pi/180]);
% L3eta  = 500*diag([5,5,pi/180]); % Use this L3 if velocity feedback is not used.
L3eta  = 50*diag([5,5,pi/180]);

% Injection from nu_tilde, if we have a velocity data series available. If
% not, then set to zero.
L1nu   = 0.1*diag([1,1,0.1*pi/180]);
L2nu   = 1*diag([1,1,0.1*pi/180]);
L3nu   = 500*diag([1,1,0.1*pi/180]);

% Start discrete simulation
for ii = 1:len
    eta = eta_data(:,ii);
    nu  = nu_data(:,ii);
    tau = tau_data(:,ii);
    eta_err    = eta-eta_hat;
    eta_err(3) = rad2radpi(eta_err(3));
    nu_err     = nu-nu_hat;

    psi = eta(3);
    R = [cos(psi) -sin(psi) 0;
         sin(psi)  cos(psi) 0;
           0         0      1];
   
   switch nx
        case 4  % Only linear damping 
            nudot = M\(tau - DL*nu_hat + b_hat );
        case 11 % Nonlinear damping and coriolis
            DNL_cross_nu = [ 0.0; 
                 DNL_cross(2,2)*abs(nu_hat(3))*nu_hat(2) + DNL_cross(2,3)*abs(nu_hat(2))*nu_hat(3);
                 DNL_cross(3,2)*abs(nu_hat(3))*nu_hat(2) + DNL_cross(3,3)*abs(nu_hat(2))*nu_hat(3) ];
            Coriolis_nu = [-M(2,2)*nu_hat(2)*nu_hat(3); 
                  M(1,1)*nu_hat(1)*nu_hat(3);
                  M(2,2)*nu_hat(2)*nu_hat(1)- M(1,1)*nu_hat(1)*nu_hat(2)]; 
            nudot = M\(tau - DL*nu_hat - DNL_diag*abs(nu_hat).*nu_hat + b_hat ...
                    -DNL_cross_nu - Coriolis_nu);
        otherwise
            nudot = M\(tau - DL*nu_hat + b_hat );
    end        
    
    b_hat_next      = b_hat   + samp_time*(L1eta*R'*eta_err + L1nu*nu_err); %-samp_time*Tb\b_hat
    eta_hat_next    = eta_hat + samp_time*(R*nu_hat + L2eta*eta_err + L2nu*R*nu_err); 
    eta_hat_next(3) = rad2radpi(eta_hat_next(3));
    nu_hat_next     = nu_hat  + nudot*samp_time + M\( L3eta*R'*eta_err + L3nu*nu_err)*samp_time;

    b_est(:,ii+1)   = b_hat_next;
    eta_est(:,ii+1) = eta_hat_next;
    nu_est(:,ii+1)  = nu_hat_next;
    
    psi = eta_hat_next(3);
    R = [cos(psi) -sin(psi) 0;
         sin(psi)  cos(psi) 0;
           0         0      1];
   
    etadot_est(:,ii+1) = R*nu_hat_next;
    time_vec(ii+1)  = samp_time*ii;

    b_hat   = b_hat_next;
    eta_hat = eta_hat_next;
    nu_hat  = nu_hat_next;
end

