#!/usr/bin/env python
import rospy
import dynamic_reconfigure.client
import numpy as np
import csv
from datetime import datetime

def callback(config):
    rospy.loginfo("Config set to {Kp_depth}, {Kd_surge}".format(**config))

def save_data(data):
    date = str(datetime.now())
    date = date.replace(' ', '_', 1)
    date = date.split('.')[0]
    date = date.replace(':', '-')  
    f = open('../controller_test/gains%s.csv' % date,'w')
    fieldnames = ['Kp_surge', 'Kd_surge', 'Kp_sway', 'Kd_sway', 'Kp_heading', 'Kd_heading', 'Kdf_heading',\
                 'Kp_depth', 'Ki_depth', 'Kd_depth', 'Kdi_depth', 'Kd_depth_rate', 'Ka_depth_acc']
    writer = csv.DictWriter(f, fieldnames=fieldnames)
    writer.writeheader()
    writer.writerow({'Kp_surge': '%f' % data[0] , 'Kd_surge': '%f' % data[1], 'Kp_sway': '%f' % data[2],\
         'Kd_sway': '%f' % data[3], 'Kp_heading': '%f' % data[4], 'Kd_heading': '%f' % data[5],\
         'Kdf_heading': '%f' % data[6], 'Kp_depth': '%f' % data[7], 'Ki_depth': '%f' % data[8],\
         'Kd_depth': '%f' % data[9], 'Kdi_depth': '%f' % data[10], 'Kd_depth_rate': '%f' % data[11],\
         'Ka_depth_acc': '%f' % data[12] })
    f.close()

if __name__ == "__main__":
    rospy.init_node("dynamic_control_param_client")
    #controller is the name of the server node
    client = dynamic_reconfigure.client.Client("controller", timeout=30, config_callback=callback)

    Kp_surge = 2
    Kd_surge = 0.1 

    Kp_sway = 2
    Kd_sway = 0.1

    Kp_heading = 0.4
    Kd_heading = 1.7
    Kdf_heading = Kd_heading 
    Ka_heading = 0

    Kp_depth = 10
    Ki_depth = 0.1
    Kd_depth = 0.5 
    Kd_depth_rate = Kd_depth 
    Kdi_depth = 0 
    Ka_depth_acc = 0 

    client.update_configuration({"Kp_surge":Kp_surge, "Kd_surge":Kd_surge, \
        "Kp_sway":Kp_sway, "Kd_sway":Kd_sway,\
        "Kp_heading": Kp_heading, "Kd_heading":Kd_heading,\
        "Kp_depth": Kp_depth, "Ki_depth":Ki_depth, "Kd_depth":Kd_depth,
        "Kdi_depth": Kdi_depth,"Kd_depth_rate":Kd_depth_rate, "Ka_depth_acc":Ka_depth_acc})

    # Save data to file:
    data_vec = np.array([Kp_surge, Kd_surge, Kp_sway, Kd_sway, Kp_heading, Kd_heading, Kdf_heading,\
        Kp_depth, Ki_depth, Kd_depth, Kdi_depth, Kd_depth_rate, Ka_depth_acc])
    save_data(data_vec)
    