# Get position from Water Linked's APS. 

from __future__ import print_function
import requests
import argparse
import json


def get_data(url):
    try:
        r = requests.get(url)
    except requests.exceptions.RequestException as exc:
        print("Exception occured {}".format(exc))
        return None

    if r.status_code != requests.codes.ok:
        print("Got error {}: {}".format(r.status_code, r.text))
        return None

    return r.json()

def get_acoustic_position(base_url):
    return get_data("{}/api/v1/position/acoustic/filtered".format(base_url))


def get_global_position(base_url):
    return get_data("{}/api/v1/position/global".format(base_url))

def get_master_orientation(base_url):
    master_data = get_data("{}/api/v1/position/master".format(base_url))
    return master_data["orientation"]

def get_APS(ip_url):
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('-u', '--url', help='Base URL to use', type=str, default='http://demo.waterlinked.com')
    args = parser.parse_args()

    base_url = ip_url #args.url
    #print("Using base_url: %s" % ip_url)

    #pos_acus = get_acoustic_position(base_url)
    #if data:
        #print(data)
        #print("Current acoustic position {},{},{}".format(data["x"], data["y"], data["z"]))

    pos_glob = get_global_position(base_url)
    #if pos:
        #print("Current global position lat:{} lon:{}".format(pos["lat"], pos["lon"]))
        
    return pos_glob


if __name__ == "__main__":
    url = 'http://demo.waterlinked.com'
    #url = 'http://192.168.0.1/'
    pos = get_APS(url)
    print("Current global position lat:{} lon:{}".format(pos["lat"], pos["lon"]))
