import numpy as np 
import config
import math
from model import Model
from getmeas_drone import DroneMeas
from getposition_aps import get_global_position, get_acoustic_position

class ObserverLuenberg:
    def __init__(self,APS):          
        self.sampT_obs = 1/config.RATE 
        self.model = Model(self.sampT_obs)
        self.x = np.zeros(9,float) 
        self.y = np.zeros(4,float)
        self.x_hat = np.zeros(9,float)
        self.drone_meas = DroneMeas()
        self.rot_z = np.zeros((3,3))

        self.L = np.zeros((9,6))
        self.L[0:3,0:3] = 100*np.diag([1,1,math.pi/180])
        self.L2eta = 500*np.diag([5,5,math.pi/180])
        self.L3eta = 0.1*np.diag([1,1,math.pi/180])
        # With velocity feedback: 
        self.L[2,3] = 0.1*0.1*math.pi/180
        self.L[5,3] = 500*0.1*math.pi/180
        self.L[8,3] = 0.1*0.1*math.pi/180 
        self.L2eta = 50*np.diag([5,5,math.pi/180])
        #Tb  = 10*np.diag([1,1,180/math.pi]) # Ta med denne ???????? 
        self.APS_offset = APS.offset
        self.angle_u2n = 0

        
    def update(self, case_model, case_parder, case_pos):
        self.get_measurements(case_pos)
        # Update model params:
        self.model.update(self.x_hat, self.sampT_obs, case_model, case_parder)
        self.rot = np.array([[math.cos(self.y[2]), -math.sin(self.y[2]), 0],\
                        [math.sin(self.y[2]), math.cos(self.y[2]), 0], \
                        [0,0,1]])
        # Observer: 
        self.L[3:6,0:3] = np.matmul(self.L2eta, np.transpose(self.rot_z))
        self.L[6:9,0:3] = np.matmul(self.L3eta, np.transpose(self.rot_z))
        #print('Transf: {}' .format(self.Transf4dof))
        y_err = self.y - np.matmul(self.model.H, self.x_hat) 
        y_err_ag = np.zeros(6) # Augmented for matrix multiplication with L
        y_err_ag[0:3] = y_err[0:3]
        y_err_ag[5] = y_err[3]

        self.L[:,0:2] = 0.0 
        self.x_hat = np.matmul(self.model.phi, self.x_hat) +\
            self.sampT_obs*np.matmul(self.model.B, self.tau) +\
                 np.matmul(self.L, y_err_ag)

    def get_measurements(self, case_pos):
        #-------GET MEASUREMENTS------------------------
        # y = [x,y,psi,heading_rate]
        self.y[2] = self.rad2radpi(self.drone_meas.state_estimate.heading*math.pi/180) #[rad]
        self.y[3] = self.drone_meas.state_estimate.heading_rate*math.pi/180  # [rad/s]
        self.tau = np.array([self.drone_meas.control_force.surge,\
            self.drone_meas.control_force.sway,\
            self.drone_meas.control_force.yaw]).T
        # x and y positiong:     
        url = 'http://192.168.0.1/'
        #User frame:
        if case_pos == 1:
            pos = get_acoustic_position(url)
            xy_u = np.array([pos["x"],pos["y"] ])
            xy_n = self.UserToNED(xy_u, url)
            xy_n = self.transform_CO(xy_n)
            self.y[0] = xy_n[0]
            self.y[1] = xy_n[1]
            
        elif case_pos == 2:
            pos = get_global_position(url)
            self.y[0] = pos["lat"]
            self.y[1] = pos["lon"]
        else:
           self.y[0] = 0
           self.y[1] = 0
        
        #print(type(pos))

    def UserToNED(self, xy_u, url):
        #Rotate 180 deg about x-axis:
        xy = np.matmul(np.diag([1,-1]),xy_u)
        #Rotate about z-axis: 
        self.angle_u2n = get_master_orientation(url)
        xy_n = np.matmul(self.rot_z(self.angle_u2n)[0:2,0:2],xy)
        return xy_n

    def transform_CO(self, xy_n):
        #Transform offset in body frame to NED. #Rotate about z:
        aps_offset = np.matmul(self.rot_z(self.y[2]), self.APS_offset)
        xy = xy_n - aps_offset[0:2]
        return xy
            
    def rot_z(self, angle):
        rot = np.array([[math.cos(angle), -math.sin(angle), 0],\
                        [math.sin(angle), math.cos(angle), 0], \
                        [0,0,1]])
        return rot

    def rad2radpi(self, x):
        # Contain angle between [-pi,pi).
        y = np.arctan2(np.sin(x), np.cos(x))
        if y == math.pi:
            y = -y
        return y

if __name__=='__main__':
    luen = ObserverLuenberg()
    luen.update(1)